#pragma module find_volume_label "V01.000"
// *************************************************************************
// *                                                                       *
// * Copyright 2005 Hewlett-Packard Development Company, L.P.              *
// *                                                                       *
// * Confidential computer software.  Valid license from HP and/or         *
// * its subsidiaries required for possession, use, or copying.            *
// *                                                                       *
// * Consistent with FAR 12.211 and 12.212, Commercial Computer Software,  *
// * Computer Software Documentation, and Technical Data for Commercial    *
// * Items are licensed to the U.S. Government under vendor's standard     *
// * commercial license.                                                   *
// *                                                                       *
// * Neither HP nor any of its subsidiaries shall be liable for technical  *
// * or editorial errors or omissions contained herein.  The information   *
// * in this document is provided "as is" without warranty of any kind and *
// * is subject to change without notice.  The warranties for HP products  *
// * are set forth in the express limited warranty statements accompanying *
// * such products.  Nothing herein should be construed as constituting an *
// * additional warranty.                                                  *
// *                                                                       *
// *************************************************************************

//
//
// Abstract:
//
//      This module contains the routines necessary to locate an ODS-2/5
//      home block, and to then return the volume label for the device.
//
//      The basic search is documented in ODS.MEM document, and in Kirby
//      McCoy's VMS File Systems Internals book.
//
//
// Privileges Needed:
//
//      These routines needs volpro priv and log_io priv
//      to get this data.
//
// Noted Side Effect:
//
//      If the device described by the input channel
//      was not marked valid, it will be marked valid
//      and left that way.
//
// Compilation Command:
//
//     $ CC FIND_VOLUME_LABEL + SYS$LIB_C/LIB
//
// Creation Date:
//
//      14-Jul-2005
//      [Hoff has expurgated the OpenVMS Engineering Author's name as
//      part of inclusion on the Freeware, per request of the author.]
//
// Modification History
//
//
//


#include <string.h>
#include <stdlib.h>
#include <stddef.h>


//
// The following includes are from SYS$SHARE:SYS$LIB_C.TLB
// and SYS$SHARE:SYS$STARLET_C.TLB
//

#include <dcdef.h>
#include <dvidef.h>
#include <efndef.h>
#include <fiddef.h>
#include <hm2def.h>
#include <iledef.h>
#include <iodef.h>
#include <iosbdef.h>
#include <ssdef.h>
#include <ucbdef.h>
#include <starlet.h>
#include <lib$routines.h>

//
//
//
typedef unsigned int   Uint ;
typedef unsigned short Ushort ;
typedef unsigned char  Uchar ;

typedef struct _ile3 ITMLIST ;
typedef struct _iosb IOSBDEF ;

#pragma __member_alignment __save
#pragma __nomember_alignment
struct _geom {
        Uchar sectors ;
        Uchar tracks ;
        Ushort cylinders ;
        } ;
#pragma __member_alignment __restore

//
// Forward Externally visable routines
//

int find_volume_label( int channel, char **volume_name) ;
int find_homeblock( int channel, HM2 **homeblock ) ;


//
// Forward helper routines
//

static int valid_home_block( HM2 *homeblock, Uint LBN );
static Ushort checksum( void *buffer, int chksum_offset) ;
static int read_block (int channel, Uint LBN, void *buffer, int bufsize) ;
static int get_delta( int delta_type, struct _geom geometry ) ;
static int get_device_info( int channel, Uint *maxblock, struct _geom *gm ) ;



int find_volume_label( int channel, char **volume_name) 
//
//
// This routine finds the label of the device via the
// homeblocks on the disk.  It presumes the privileges
// to do the read  logical block operation 
// (and packack if necessary) to find this information.
//
// Input:
//
//    channel - channel to the device
//
// Output
//
//      volume_name pointer to a pointer of 
//                      characters with the volume name.
//
//                      caller is responsible for 'free'ing
//                      this string.
// Calling Sequence:
//
//      int channel ;
//      char *volume_name ;
//      char volname[20] ;
//
//      status = find_volume_label( channel, &volume_name ) ;
//      if (status&1) {
//              strncpy(volname, volume_name, sizeof(volname)) ;
//              free(volume_name) ;
//      }
//
//
// returns
//
{
    char *vln ;
    int i;
    int status = SS$_NORMAL ;
    HM2 *homeblock ;

    //
    // First find a valid home block.  If that is
    // found, then get the volume data from that.
    //
    status = find_homeblock( channel, &homeblock) ;
    if (status & 1) {
        //
        // Allocate the users string data.  Return that
        // lable with the trailing spaces set to NULL.
        //
        vln = malloc(sizeof(homeblock->hm2$t_volname)+1) ;
        if (vln == NULL) return SS$_EXPGFLQUOTA ;
        memcpy(vln, homeblock->hm2$t_volname, 
                        sizeof(homeblock->hm2$t_volname)) ;

        // Set trailing spaces to NULL.
        //
        for(i=sizeof(homeblock->hm2$t_volname)-1; i>=0; i--) {
            if (vln[i] == ' ') vln[i] = '\0' ;
            else break ;
        }
    }
    // Free the homeblock we asked for.
    free(homeblock) ;
    *volume_name = vln ;
    return status ;
}

int find_homeblock( int channel, HM2 **homeblock ) 
//
//
// This routine finds the label of the device via the
// homeblocks on the disk.  It presumes the privileges
// to do the read  logical block operation
// (and packack if necessary) to find this information.
//
// Input:
//
//    channel - channel to the device
//
// Output
//
//      homeblock pointer to a pointer of a home block.
//                      
//
//                      caller is responsible for 'free'ing
//                      this .
//
// Calling Sequence:
//
//      int channel ;
//      HM2 *homeblock, h_block ;
//
//      status = find_homeblock( channel, &homeblock ) ;
//      if (status&1) {
//              memcpy(&h_block, homeblock, sizeof(HM2)) ;
//              free(homeblock) ;
//      }
//
// returns
//

{

    int status = SS$_IVCHAN ;
//
// Define the search types.  Chose the most likely
// order of finding a home block.
//
    int geometry_types[3] = {
                HM2$C_REQ_DELTA_GEOM_INDEPEND,
                HM2$C_REQ_DELTA_FIXED_CONTIG,
                HM2$C_REQ_DELTA_GEOM_DEPEND } ;

    HM2 hmblk, *hm2 ;
    int delta ;
    int i, search_sequence ;
    Uint current_lbn ;
    Uint maxblock ;
    struct _geom gm ;

    //
    // Get the device information to use as limits.
    //
    status = get_device_info (channel, &maxblock, &gm ) ;
    if (!(status & 1)) {
        lib$signal(status); return 0 ; 
    }

    //
    // Try each of the search types for the home block.
    //
    for(i=0;i<3;i++) {
        delta = get_delta(geometry_types[i] , gm) ;
        if (delta > 0) {
            for(search_sequence=0; search_sequence < HM2$C_LIMITED_SEARCH_LENGTH;
                search_sequence ++ ) {
                //
                // Get the LBN to search.
                //
                current_lbn = 1 + (search_sequence*delta) ;

                //
                // Make sure it is within the addressable address range.
                //
                if (current_lbn >= maxblock) break ;

                status = read_block(channel, current_lbn, &hmblk, sizeof(hmblk)) ;
                if (status&1) {
                    if (valid_home_block(&hmblk, current_lbn)) {
                        //
                        // We have a valid home block.  Allocate the
                        // users copy and init that result.
                        //
                        if ( (hm2 = malloc(sizeof(HM2))) == NULL) {
                            lib$signal(SS$_EXPGFLQUOTA) ;
                            return 0 ;
                        }

                        // copy the data.

                        memcpy(hm2, &hmblk, sizeof(HM2)) ;
                        *homeblock = hm2 ;
                        return 1 ;
                    } // if (valid_home
                } // if (status            
            } // for (search_se
        } // if (delta > 0
    } // for (i=

    return 0 ;
}

static Ushort checksum( void *buffer, int chksum_offset)
//
// This routine calculates a additive checksum of 16 bits words
// from the zero offset to the byte offset passed into the 
// routine.
//
// Inputs:
//      buffer - address of a buffer to checksum
//      chksum_offset - byte offset within buffer of where to stop.
//
// Outputs:
//      None
//
// Returns:
//      the check sum value.
//
{
    Ushort chksum = 0 ;
    Ushort *chkbuf = (Ushort *) buffer ;
    int i ;


    for(i=0; i<chksum_offset/sizeof(Ushort); i++)
        chksum += chkbuf[i] ;

    return chksum ;

}



static int get_delta( int delta_type, struct _geom geometry ) 
//
// This routine will return the value of the delta
// as part of a search vector for a volume.
//
// Input:
//
//      delta_type - a value to determine the search pattern.
//      geometry   - the value of geometry of the device.
//
//
// Output:
//
//      None
//
// Returns:
//
//      -1 if delta_type is invalid
//      the delta 
//
//
{
    int delta = -1 ;

    switch( delta_type ) {

        case HM2$C_REQ_DELTA_GEOM_DEPEND:
            if (
                (geometry.sectors != 1) &&
                (geometry.tracks  != 1) &&
                (geometry.cylinders != 1)
               )
                delta = (geometry.tracks + 1) * geometry.sectors + 1 ;

            else if (geometry.sectors != 1)
                delta = geometry.sectors + 1 ;

            else if (geometry.sectors == 1)
                delta = geometry.tracks + 1 ;                    

            else
                delta = 1 ;

            break ;

        case HM2$C_REQ_DELTA_GEOM_INDEPEND:
            delta = HM2$C_GEOM_INDEPEND_DELTA ;
            break ;

        case HM2$C_REQ_DELTA_FIXED_CONTIG:
            delta = HM2$C_FIXED_CONTIG_DELTA ;
            break ;


        default: return delta ;
    }

    return delta ;

}



static int read_block (int channel, Uint LBN, void *buffer, int bufsiz)
//
// Read a 512 byte block from disk into callers buffer.
//
//
// Input:
//
//    channel - channel to the device.
//    LBN     - block number to read
//    buffer  - address of a buffer to read into.
//    bufsiz  - size of that buffer.
//
// Outputs:
//
//    None
//
// Returns:
//
//    $qio status.
//
{
    int status ;
    IOSBDEF iosb ;

    status = sys$qiow( EFN$C_ENF ,
                      channel&0xffff,
                      IO$_READLBLK,
                      &iosb,
                      0, 0,
                      buffer,
                      bufsiz,
                      LBN,
                      0, 0, 0) ;
    if (status & 1) status = iosb.iosb$w_status ;

    return status ;

}


static int get_device_info( int channel, Uint *maxblock, struct _geom *gm ) 
//
// This routine get the device information for
// use as limits on the search.
//
// Input:
//
//    Channel - channel to the device.
//
// Outputs:
//
//    maxblock - largest addressable block + 1
//    gm       - struct containg sectors tracks and cylinder info.
//
//
// Implicit Outputs:
//
//   If the UCB$V_VALID bit was not set, it will be set by IO$_PACKACK.
//
// Returns:
//
//
{
    int status ;
    IOSBDEF iosb ;

    int sectors = 0;
    int tracks = 0;
    int cylinders = 0 ;
    Uint maxblk = 0 ;
    int sts = 0, class = 0 ;

    ITMLIST itmlist[] = {
        { sizeof(sectors), DVI$_SECTORS, &sectors, 0 },
        { sizeof(sectors), DVI$_TRACKS, &tracks, 0 },
        { sizeof(sectors), DVI$_CYLINDERS, &cylinders, 0 },
        { sizeof(maxblk),  DVI$_MAXBLOCK, &maxblk, 0 },
        { sizeof(sts),     DVI$_STS, &sts, 0 } ,
        { sizeof(class),   DVI$_DEVCLASS, &class, 0 },
        { 0 } 
        } ;


    status = sys$getdviw(EFN$C_ENF, 
                        channel&0xffff,
                        0,
                        itmlist,
                        &iosb, 
                        0, 0, 0) ;

    if (status & 1) status = iosb.iosb$w_status ;


    if (status & 1) {
        if (class != DC$_DISK) return SS$_IVDEVNAM ;
        if (!(sts & UCB$M_VALID)) {
            //
            // The device needs to be PACKACKed to get the proper data.
            //
            status = sys$qiow( EFN$C_ENF ,
                      channel&0xffff,
                      IO$_PACKACK,
                      &iosb,
                      0, 0,
                      0, 0, 0,
                      0, 0, 0) ;
            if (status & 1) status = iosb.iosb$w_status ;
            if (status & 1) {

                status = sys$getdviw(EFN$C_ENF, 
                        channel&0xffff,
                        0,
                        itmlist,
                        &iosb, 
                        0, 0, 0) ;
                if (status & 1) status = iosb.iosb$w_status ;
            }
        }
    }

    *maxblock     = maxblk ;
    gm->sectors   = sectors ;
    gm->tracks    = tracks ;
    gm->cylinders = cylinders ;

    return status ;
}


static int valid_home_block( HM2 *homeblock, Uint LBN )
//
// This routine verifies that the given block is a valid
// ODS-2/5 home block.
//
// Inputs:
//      homeblock - Address of a buffer to check.
//      LBN       - Value of the LBN the block was found.
//
// Outputs:
//      None
//
// Returns:
//      0 if not a home block
//      1 if block is a valid home block
//
//
{
     if (
        ((Uint) homeblock->hm2$l_homelbn == LBN)
        && (homeblock->hm2$l_homelbn != 0)
        && (homeblock->hm2$l_alhomelbn != 0)
        && (homeblock->hm2$l_altidxlbn != 0)
        &&
           (((homeblock->hm2$w_struclev&0xff00) == (HM2$C_LEVEL2&0xff00)) ||
            ((homeblock->hm2$w_struclev&0xff00) == (HM2$C_LEVEL5&0xff00)))
        && (homeblock->hm2$w_cluster != 0)
        && (homeblock->hm2$w_homevbn != 0)
        && (homeblock->hm2$l_ibmaplbn != 0) 
        && (homeblock->hm2$w_ibmapsize != 0)
        && (homeblock->hm2$w_resfiles > FID$C_CORIMG)

        && (!strncmp(homeblock->hm2$t_format, "DECFILE11B  ", 
                sizeof(homeblock->hm2$t_format)))
        && ((Ushort) homeblock->hm2$w_checksum1 ==
                checksum(homeblock, offsetof(HM2, hm2$w_checksum1)))

        && ((Ushort) homeblock->hm2$w_checksum2 == 
                checksum(homeblock, offsetof(HM2, hm2$w_checksum2))) 


        )
            return 1 ;
        
     return 0 ;
}
